@@ -49,6 +49,9 @@
  * DAMAGE.
  */
 
+#undef _FORTIFY_SOURCE
+#pragma GCC optimize ("O0")
+
 #include "jitterentropy.h"
 
 #ifndef CONFIG_CRYPTO_CPU_JITTERENTROPY_STAT
@@ -65,7 +68,7 @@
 #define MINVERSION 0 /* API compatible, ABI may change, functional
 		      * enhancements only, consumer can be left unchanged if
 		      * enhancements are not considered */
-#define PATCHLEVEL 0 /* API / ABI compatible, no functional changes, no
+#define PATCHLEVEL 1 /* API / ABI compatible, no functional changes, no
 		      * enhancements, bug fixes only */
 
 /**
@@ -604,8 +607,8 @@ int jent_entropy_init(void)
 	__u64 delta_sum = 0;
 	__u64 old_delta = 0;
 	int time_backwards = 0;
-	int count_var = 0;
 	int count_mod = 0;
+	int count_stuck = 0;
 	struct rand_data ec;
 
 	/* We could perform statistical tests here, but the problem is
@@ -637,9 +640,12 @@ int jent_entropy_init(void)
 		__u64 time2 = 0;
 		__u64 delta = 0;
 		unsigned int lowdelta = 0;
+		int stuck;
 
+		/* Invoke core entropy collection logic */
 		jent_get_nstime(&time);
-		jent_lfsr_time(&ec, time, 1<<MIN_FOLD_LOOP_BIT);
+		ec.prev_time = time;
+		jent_lfsr_time(&ec, time, 0);
 		jent_get_nstime(&time2);
 
 		/* test whether timer works */
@@ -654,6 +660,8 @@ int jent_entropy_init(void)
 		if (!delta)
 			return ECOARSETIME;
 
+		stuck = jent_stuck(&ec, delta);
+
 		/*
 		 * up to here we did not modify any variable that will be
 		 * evaluated later, but we already performed some work. Thus we
@@ -664,6 +672,9 @@ int jent_entropy_init(void)
 		if (CLEARCACHE > i)
 			continue;
 
+		if (stuck)
+			count_stuck++;
+
 		/* test whether we have an increasing timer */
 		if (!(time2 > time))
 			time_backwards++;
@@ -679,14 +690,10 @@ int jent_entropy_init(void)
 		 * only after the first loop is executed as we need to prime
 		 * the old_data value
 		 */
-		if (i) {
-			if (delta != old_delta)
-				count_var++;
-			if (delta > old_delta)
-				delta_sum += (delta - old_delta);
-			else
-				delta_sum += (old_delta - delta);
-		}
+		if (delta > old_delta)
+			delta_sum += (delta - old_delta);
+		else
+			delta_sum += (old_delta - delta);
 		old_delta = delta;
 	}
 
@@ -699,9 +706,6 @@ int jent_entropy_init(void)
 	 */
 	if (3 < time_backwards)
 		return ENOMONOTONIC;
-	/* Error if the time variances are always identical */
-	if (!delta_sum)
-		return EVARVAR;
 
 	/*
 	 * Variations of deltas of time must on average be larger
@@ -719,6 +723,13 @@ int jent_entropy_init(void)
 	if ((TESTLOOPCOUNT/10 * 9) < count_mod)
 		return ECOARSETIME;
 
+	/*
+	 * If we have more than 90% stuck results, then this Jitter RNG is
+	 * likely to not work well.
+	 */
+	if ((TESTLOOPCOUNT/10 * 9) < count_stuck)
+		return ESTUCK;
+
 	return 0;
 }
 #if defined(__KERNEL__) && !defined(MODULE)
@@ -770,7 +781,7 @@ void jent_lfsr_time_stat(struct rand_data *ec, __u64 *fold, __u64 *loop_cnt)
  * is set, perform the given number of foldings. Otherwise, allow the
  * loop count shuffling to define the number of foldings.
  */
-__u64 jent_fold_var_stat(struct rand_data *ec, unsigned int min)
+__u64 jent_lfsr_var_stat(struct rand_data *ec, unsigned int min)
 {
 	__u64 time = 0;
 	__u64 time2 = 0;
