@@ -61,7 +61,7 @@
 #define MINVERSION 0 /* API compatible, ABI may change, functional
 		      * enhancements only, consumer can be left unchanged if
 		      * enhancements are not considered */
-#define PATCHLEVEL 5 /* API / ABI compatible, no functional changes, no
+#define PATCHLEVEL 6 /* API / ABI compatible, no functional changes, no
 		      * enhancements, bug fixes only */
 
 static int Verbosity = 0;
@@ -164,6 +164,8 @@ static void parse_opts(int argc, char *argv[])
 			case 3:
 				jentrng_versionstring(version, sizeof(version));
 				fprintf(stderr, "Version %s\n", version);
+				fprintf(stderr, "Version Jitterentropy Core %u\n", jent_version());
+				exit(0);
 				break;
 			default:
 				usage();
@@ -545,18 +547,22 @@ static void daemonize(void)
 		create_pid_file(Pidfile);
 
 	/* Redirect standard files to /dev/null */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-result"
 	freopen( "/dev/null", "r", stdin);
 	freopen( "/dev/null", "w", stdout);
 	freopen( "/dev/null", "w", stderr);
+#pragma GCC diagnostic pop
 }
 
 
 int main(int argc, char *argv[])
 {
+	parse_opts(argc, argv);
+
 	if (geteuid())
 		dolog(LOG_ERR, "Program must start as root!");
 
-	parse_opts(argc, argv);
 	alloc();
 	if (0 == Verbosity)
 		daemonize();
